<xsl:transform version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:my="http://xml.com/my-template-language"
>

<xsl:param name="template-uri" />

<xsl:variable name="template" select="document($template-uri)" />
<xsl:variable name="source" select="/" />

<xsl:template match="/">
	<xsl:apply-templates select="$template/node()" />
</xsl:template>

<xsl:template match="*">
	<xsl:param name="current-album" />
	<xsl:param name="current-artist" />
	<xsl:element name="{name()}">
		<xsl:apply-templates select="@* | node()">
			<xsl:with-param name="current-album" select="$current-album" />
			<xsl:with-param name="current-artist" select="$current-artist" />
		</xsl:apply-templates>
	</xsl:element>
</xsl:template>

<xsl:template match="@*">
	<xsl:attribute name="{name()}">
		<xsl:call-template name='attribute'>
			<xsl:with-param name='value' select='string(.)' />
		</xsl:call-template>
	</xsl:attribute>
</xsl:template>

<xsl:template name="attribute">
	<xsl:param name="value" />
	<xsl:choose>
		<xsl:when test="contains($value, '{$')">
			<xsl:value-of select="substring-before($value, '{$')" />
			<xsl:variable name="name" select="substring-before(substring-after($value, '{$'), '}')" />
			<xsl:variable name="node" select="document('')/xsl:transform/my:*[local-name() = $name]" />
			<xsl:choose>
				<xsl:when test="$node">
					<xsl:apply-templates select="$node" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="concat('{$', $name, '}')" />
				</xsl:otherwise>
			</xsl:choose>
			<xsl:call-template name="attribute">
				<xsl:with-param name="value" select="substring-after(substring-after($value, '{$'), '}')" />
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$value" />
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<my:owner-email />

<xsl:template match="text()">
	<xsl:if test="normalize-space()">
		<xsl:value-of select="." />
	</xsl:if>
</xsl:template>

<xsl:template match="my:text">
	<xsl:value-of select="." />
</xsl:template>

<xsl:template match="my:owner-name">
	<xsl:value-of select="$source/collection/owner/name" />
</xsl:template>

<xsl:template match="my:owner-email">
	<xsl:value-of select="$source/collection/owner/email" />
</xsl:template>

<xsl:template match="my:if-owner-has-email">
	<xsl:if test="$source/collection/owner/email">
		<xsl:apply-templates />
	</xsl:if>
</xsl:template>

<xsl:template match="my:for-each-album">
	<xsl:variable name="current" select="." />
	<xsl:for-each select="$source/collection/album">
		<xsl:apply-templates select="$current/node()">
			<xsl:with-param name="current-album" select="." />
		</xsl:apply-templates>
	</xsl:for-each>
</xsl:template>

<xsl:template match="my:album-artist">
	<xsl:param name="current-album" />
	<xsl:value-of select="$current-album/artist" />
</xsl:template>

<xsl:template match="my:album-title">
	<xsl:param name="current-album" />
	<xsl:value-of select="$current-album/title" />
</xsl:template>

<xsl:template match="my:for-each-artist">
	<xsl:variable name="current" select="." />
	<xsl:for-each select="$source/collection/album[not(artist = preceding-sibling::album/artist)]/artist">
		<xsl:apply-templates select="$current/node()">
			<xsl:with-param name="current-artist" select="." />
		</xsl:apply-templates>
	</xsl:for-each>
</xsl:template>

<xsl:template match="my:artist-name">
	<xsl:param name="current-artist" />
	<xsl:value-of select="$current-artist" />
</xsl:template>

<xsl:template match="my:for-each-album-by-artist">
	<xsl:param name="current-artist" />
	<xsl:variable name="current" select="." />
	<xsl:for-each select="$source/collection/album[artist = $current-artist]">
		<xsl:apply-templates select="$current/node()">
			<xsl:with-param name="current-artist" select="$current-artist" />
			<xsl:with-param name="current-album" select="." />
		</xsl:apply-templates>
	</xsl:for-each>
</xsl:template>

<xsl:template match="my:choose">
	<xsl:param name="current-album" />
	<xsl:param name="current-artist" />
	<xsl:call-template name="choose">
		<xsl:with-param name="choices" select="*" />
		<xsl:with-param name="current-album" select="$current-album" />
		<xsl:with-param name="current-artist" select="$current-artist" />
	</xsl:call-template>
</xsl:template>

<xsl:template name="choose">
	<xsl:param name="choices" />
	<xsl:param name="current-album" />
	<xsl:param name="current-artist" />
	<xsl:choose>
		<xsl:when test="not($choices)" />
		<xsl:when test="local-name($choices) != 'otherwise'">
			<xsl:variable name="result">
				<xsl:apply-templates select="$choices[1]">
					<xsl:with-param name="current-album" select="$current-album" />
					<xsl:with-param name="current-artist" select="$current-artist" />
				</xsl:apply-templates>
			</xsl:variable>
			<xsl:choose>
				<xsl:when test="$result = 'true'">
					<xsl:apply-templates select="$choices[1]/node()">
						<xsl:with-param name="current-album" select="$current-album" />
						<xsl:with-param name="current-artist" select="$current-artist" />
					</xsl:apply-templates>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="choose">
						<xsl:with-param name="choices" select="$choices[position() > 1]" />
						<xsl:with-param name="current-album" select="$current-album" />
						<xsl:with-param name="current-artist" select="$current-artist" />
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:otherwise>
			<xsl:apply-templates select="$choices[1]/node()">
				<xsl:with-param name="current-album" select="$current-album" />
				<xsl:with-param name="current-artist" select="$current-artist" />
			</xsl:apply-templates>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="my:when-album-has-title">
	<xsl:param name="current-album" />
	<xsl:value-of select="boolean($current-album/title)" />
</xsl:template>

<xsl:template match="my:for-each-album[@sort-by]">
	<xsl:variable name="sort-by" select="@sort-by" />
	<xsl:variable name="current" select="." />
	<xsl:for-each select="$source/collection/album">
		<xsl:sort select="*[local-name() = $sort-by]" />
		<xsl:apply-templates select="$current/node()">
			<xsl:with-param name="current-album" select="." />
		</xsl:apply-templates>
	</xsl:for-each>
</xsl:template>

</xsl:transform>
